# RxIO
# by Eresse <eresse@eresse.net>

# RxIO Module
module RxIO

	# Handler Base Module:
	# Provides common abstractions to Service Handler implementations.
	module HandlerBase

		# Write:
		# Writes one or more chunks of data to the endpoint's output buffer (:obuf).
		# @param [Hash] endpoint
		# @param [String] data One or more chunks of data to be written to the ouput buffer
		def write endpoint, *data

			# Add Data Chunks to Buffer
			data.each { |c| endpoint[:lock].synchronize { endpoint[:obuf] << c.to_s } }
		end

		# Buffer Input Chunk:
		# Writes a chunk of data to the endpoint's input buffer (:ibuf).
		# @param [Hash] endpoint
		# @param [String] chunk
		def buffer_input endpoint, chunk

			# Buffer Chunk
			endpoint[:ibuf] << chunk
		end
	end
end
