# RxIO
# by Eresse <eresse@eresse.net>

# Internal Includes
require 'rxio/handler_base'

# RxIO Module
module RxIO

	# I/O Filters Module
	module IOFilters

		# Binary-Delimiter I/O Filter:
		# Splits messages according to a given fixed *binary delimiter*, which can be any number of bytes, defined through the _msg_delim_ method.
		module BinDelim

			# Inject Dependencies into Extending Module
			# @param [Module] base
			def self.extended base
				base.extend RxIO::HandlerBase
			end

			# Set Message Delimiter:
			# Used to define the binary string used as message delimiter for this protocol.
			# @param [String] v The message delimiter string
			def msg_delim v
				@msg_delim = v
			end

			# Filter Input:
			# Buffers data chunks sent by the endpoint and extracts messages from them, according to the delimiter defined through _msg_delim_.
			# @param [Hash] endpoint
			# @param [String] chunk
			def filter_input endpoint, chunk

				# Buffer dat shit
				buffer_input endpoint, chunk

				# Ensure Last Position is available
				endpoint[:bin_delim] ||= {}
				endpoint[:bin_delim][:last_pos] ||= 0

				# Loop over Messages
				while true

					# Find Delimiter
					d = endpoint[:ibuf].index @msg_delim, endpoint[:bin_delim][:last_pos]
					endpoint[:bin_delim][:last_pos] = endpoint[:ibuf].bytesize

					# Check Delimiter
					break unless d

					# Slice out Message from Input Buffer
					m = endpoint[:ibuf].slice!(0, d + @msg_delim.bytesize).slice 0, d
					endpoint[:bin_delim][:last_pos] = 0

					# Register Message
					endpoint[:msgs] << m
				end
			end

			# Send Message:
			# Buffers a message to be sent to the endpoint, after wrapping it according to the delimiter defined through _msg_delim_.
			# @param [Hash] endpoint
			# @param [String] msg
			def send_msg endpoint, msg
				write endpoint, msg, @msg_delim
			end
		end
	end
end
